# Synopsis

### What is TReqS

TReqS stands for Tape Request Scheduler. 
It's a software companion to HPSS, that re-organize HPSS staging requests to mainly

- decrease number of tape movements, by grouping files from same tape
- increase staging throughput, by re-ordering files to be staged from same tape



### What is NOT TReqS 

TReqS is **not** a file transfer protocol **nor** a storage system **nor** even a HPSS copy utility. 

It only allows to schedule staging requests in order to make HPSS files available on HPSS disk. 

You then have to use your own transfer protocol to copy the file on your local disk.


### Architecture

TReqS is a client-server application.

Server, written in java, provides a REST web service so you should be able to communicate 
with it with a simple HTTP client like cURL (http://curl.haxx.se/)
The REST API is detailed on the TReqS web site.


To avoid all the JSON and HTTP stuff, we provide a TReqS client in Python


### Requirements


- Java (JDK-1.7 recommended)
- JMS server, .e.g Oracle MessageQueue 5.1 
- TReqS server and optionally TReqS client- 
- HPPS lib, include and access

# Installation

- JMS server, .e.g Oracle MessageQueue 5.1 
	
	To download the Oracle MessageQueue 5.1 software:
	
	```
	curl -O http://download.java.net/mq/open-mq/5.1/latest/openmq5_1-binary-linux.zip
	unzip openmq5_1-binary-linux.zip
	```
	To start | stop the JMS server
	
	```
	nohup MessageQueue5.1/mq/bin/imqbrokerd -bgnd -silent &
	killall imqbrokerd
	```

	To check the JMS server
	
	```
	telnet localhost 7676
	
	Trying 127.0.0.1...
	Connected to localhost.
	Escape character is '^]'.
	101 imqbroker 301
	portmapper tcp PORTMAPPER 7676 [sessionid=5534733500552585216]
	cluster_discovery tcp CLUSTER_DISCOVERY 0
	```
		
- TReqS server

	To download the TReqS server (unzip the archive)

	```
	http://maven.in2p3.fr/fr/in2p3/cc/storage/treqs/treqs-delivery/
	```
	
	Files are organized as follows:
	
	```
	<TREQS-HOME>
	  |-bin                : binaries
	  |-etc                : configuration files
	  |-lib                : libraries
	  |-logs               : logs
	  |-data               : database
	  |-templates          : HTML templates
	   
	```
	
	To start | stop the TReqS server
	
	```
	<TREQS-HOME>/jsw/treqsd/bin/treqsd start | stop 
	```

	To check the TReqS server
	
	```
	<TREQS-HOME>/jsw/treqsd/bin/treqsd status
	
	curl http://localhost:8080/treqs2/info/ping
	```
	
- HPPS stuff 

	```
	Comming soon !
	```



- TReqS client

	To download the TReqS client (unzip the archive)



	```
	https://gitlab.in2p3.fr/cc-in2p3-dev/treqs2-client
	
	```


# Configuration


### Authentication & authorization

Apart from the 'ping' command, all TReqS endpoints are authenticated.

TReqS comes with a default user 'treqs' with the password 'changeit'. 

To change the password of the default 'treqs' user to the new value 'NEWPASSWORD':

```
> curl -X PUT 'http://treqs:changeit@localhost:8080/treqs2/security/authentication/user/treqs/password/NEWPASSORD'
```

To create a new user 'me' with password 'mysecret' 

```
> curl -X POST -H 'Content-Type':'application/json' \
    -d '{"username": "me", "password": "mysecret"}' \
    http://treqs:changeit@localhost:8080/treqs2/security/authentication/user/
```
You can also do this with the TReqS client.

### Tape models

Before your TReqS instance is able to schedule requests, 
you need to define a set of models of tapes.

Each of them with specific information like 
- the name
- the maximum number of parallel drives allocated to this model 
- the average READ rate in MiB/s. 

TReqS will not be able to stage a file which is located on a tape 
whose model is not defined in the system.

To create a new model named 'T10K-D' with a reading rate of 100MiB/s and 
allocate a maximum of 10 drives:

```
> curl -X POST -H 'Content-Type':'application/json' \
    -d '{"name": "T10K-D", "max_parallel_staging": "10", "reading_rate": "100"}' \
    http://treqs:changeit@localhost:8080/treqs2/info/tapemodel 
```
You can also do this with the TReqS client.

### Properties

TReqS is configured by default, but you can overwrite parameters in etc/treqs2.properties

For example to set H2 DB data root directory into /var/h2/ you will use treqs.db.root_dir property like this : 

```
treqs.db.root_dir=/var/h2/
```


Properties prefixed with 'treqs.dynamic.' can be changed at run-time, cool !

Other properties require to restart the server.


# Quick start

### Submitting staging requests
#### 1. Submit a staging request

```
> curl -X POST -H 'Content-Type':'application/json' \
    -d '{"file" : {"filename": "/path/to/my/file"}}' \
    http://treqs:changeit@localhost:8080/treqs2/staging/request
```

#### 2. Get the request ID:
If the submit request succeeds, you will get the ID in the response .e.g. 

 "id":"68a87adc-ba11-4be7-9c3d-e901d1b7609a"

#### 3. Query the status of the request (second request for a prettier display)
```
> curl http://treqs:changeit@localhost:8080/treqs2/staging/request/68a87adc-ba11-4be7-9c3d-e901d1b7609a

> curl -s http://treqs:changeit@localhost:8080/treqs2/staging/request/68a87adc-ba11-4be7-9c3d-e901d1b7609a | python -m json.tool
```

As soon as the request becomes successful, the file is available on HPSS disk.
A successful request has status=ENDED and sub_status=SUCCEEDED.
The file can be:
- status=ENDED, sub_status=ALREADYONDISK if the file was already on disk
- status=ENDED, sub_status=STAGED if the file was staged from HPSS

You can also do this with the TReqS client.

### Log files
Logs are provided by using Logback

By default, logs are located in 'logs/treqs2.log'. 
Log destination and other parameters like log level can be changed in 'etc/logback.xml'.


### Monitoring

TReqS provides some useful monitoring web pages

Point your browser to **http://localhost:8080/treqs2/monitoring/requests** using an authenticated user, e.g. 'treqs', and enjoy ;-) 


# License

TReqS is a free software 
You can redistribute it and/or modify it under the terms of the GNU Lesser General Public License 

See COPYING and COPYING.LESSER files for details